# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterToHeaderTrait import AddParameterToHeaderTrait
from pohoda.entity.contract.Desc import Desc


class Contract(Agenda, AddParameterToHeaderTrait):
    import_root = 'lCon:contract'

    def __init__(self, data: dict, ico: str):
        data = {
            'header': Desc(data, ico)
        }
        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('contract', namespace='con')
        xml.set('version', '2.0')
        self._add_elements(xml, ['header'], 'con')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.type.Address import Address


class Header(Agenda, AddParameterTrait):
    _ref_elements = ['number', 'store', 'centreSource', 'centreDestination', 'activity', 'contract']
    _elements = ['number', 'date', 'time', 'dateOfReceipt', 'timeOfReceipt', 'symPar', 'store', 'text',
                 'partnerIdentity', 'centreSource', 'centreDestination', 'activity', 'contract', 'note', 'intNote']

    def __init__(self, data: dict, ico: str):
        # process partner identity
        partner_identity = data.get('partnerIdentity')
        if partner_identity:
            data['partnerIdentity'] = Address(partner_identity, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('prevodkaHeader', namespace='pre')
        self._add_elements(xml, self._elements + ['parameters'], 'pre')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterToHeaderTrait import AddParameterToHeaderTrait
from pohoda.entity.int_doc.Header import Header
from pohoda.entity.int_doc.Item import Item
from pohoda.entity.int_doc.Summary import Summary


class IntDoc(Agenda, AddParameterToHeaderTrait):
    import_root = 'lst:intDoc'

    def __init__(self, data: dict, ico: str):
        data = {
            'header': Header(data, ico)
        }
        super().__init__(data, ico)

    def add_item(self, data: dict) -> 'IntDoc':
        """
        Add intDoc item.
        :param data:
        :return:
        """

        if not self._data.get('intDocDetail'):
            self._data['intDocDetail'] = []

        self._data['intDocDetail'].append(Item(data, self._ico))
        return self

    def add_summary(self, data: dict) -> 'IntDoc':
        """
        Add intDoc summary.
        :param data:
        :return:
        """

        self._data['summary'] = Summary(data, self._ico)
        return self

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('intDoc', namespace='int')
        xml.set('version', '2.0')
        self._add_elements(xml, ['header', 'intDocDetail', 'summary'], 'int')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.type.StockItem import StockItem


class Item(Agenda):
    _elements = ['quantity', 'stockItem', 'note']

    def __init__(self, data: dict, ico: str):
        # process stock item
        stock_item = data.get('stockItem')
        if stock_item:
            data['stockItem'] = StockItem(stock_item, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('prevodkaItem', namespace='pre')
        self._add_elements(xml, self._elements, 'pre')
        return xml



from lxml import etree
from pohoda.entity.document.Part import Part
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.type.Address import Address
from pohoda.entity.type.MyAddress import MyAddress


class Header(Part, AddParameterTrait):

    def __init__(self, data: dict, ico: str):
        partner_identity = data.get('partnerIdentity')
        if partner_identity:
            data['partnerIdentity'] = Address(partner_identity, ico)

        # process my identity
        my_identity = data.get('myIdentity')
        if my_identity:
            data['myIdentity'] = MyAddress(my_identity, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:

        if not self._namespace:
            raise ValueError('Namespace not set.')

        if not self._node_prefix:
            raise ValueError('Node name prefix not set.')

        xml = self._create_xml_tag('{}Header'.format(self._node_prefix), namespace=self._namespace)
        self._add_elements(xml, self._elements + ['parameters'], self._namespace)
        return xml



# coding: utf-8
import html
from lxml import etree
from pohoda.entity.Agenda import Agenda


class Parameter(Agenda):
    _namespace = None

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('parameter', namespace='typ')
        child = self._create_xml_tag('name', namespace='typ')
        child.text = self._data['name']
        xml.append(child)

        if self._data['type'] == 'list':
            self._add_ref_element(xml, 'listValueRef', self._data['value'], namespace='typ')
            if self._data['list']:
                self._add_ref_element(xml, 'list', self._data['list'], namespace='typ')
            return xml

        not_list_child = self._create_xml_tag(self._data['type'] + 'Value', namespace='typ')
        not_list_child.text = html.escape(str(self._data['value']))
        xml.append(not_list_child)
        return xml



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class Filter(Agenda):
    _ref_elements = ['storage', 'store']
    _elements = ['id', 'code', 'EAN', 'name', 'storage', 'store', 'internet', 'company', 'ico', 'dic', 'lastChanges',
                 'dateFrom', 'dateTill']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('filter', namespace='ftr')
        self._add_elements(xml, self._elements, 'ftr')
        return xml



# coding: utf-8

from pohoda.entity.common.Trait import Trait


class SetNamespaceTrait(Trait):
    _namespace = None

    def set_namespace(self, namespace: str) -> None:
        self._namespace = namespace



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class Price(Agenda):
    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('stockPriceItem', namespace='stk')
        return self._add_ref_element(xml, 'stockPrice', {
            'ids': self._data['code'],
            'price': self._data['value']
        }, namespace='stk')



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.type.Address import Address
from pohoda.entity.type.MyAddress import MyAddress


class Header(Agenda, AddParameterTrait):
    _ref_elements = ['extId', 'number', 'paymentType', 'priceLevel', 'centre', 'activity', 'contract', 'regVATinEU',
                     'MOSS', 'evidentiaryResourcesMOSS', 'carrier']
    _elements = ['orderType', 'number', 'numberOrder', 'date', 'dateDelivery', 'dateFrom', 'dateTo', 'text',
                 'partnerIdentity', 'myIdentity', 'paymentType', 'priceLevel', 'isExecuted', 'isReserved', 'centre',
                 'activity', 'contract', 'regVATinEU', 'MOSS', 'evidentiaryResourcesMOSS', 'accountingPeriodMOSS',
                 'note', 'carrier', 'intNote', 'markRecord']

    def __init__(self, data: dict, ico: str):
        # process partner identity
        partner_identity = data.get('partnerIdentity')
        if partner_identity:
            data['partnerIdentity'] = Address(partner_identity, ico)

        # process my identity
        my_identity = data.get('myIdentity')
        if my_identity:
            data['myIdentity'] = MyAddress(my_identity, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('orderHeader', namespace='ofr')
        self._add_elements(xml, self._elements + ['parameters'], 'ofr')
        return xml



# coding: utf-8

from pohoda.entity.common.Trait import Trait


class SetNodeNameTrait(Trait):
    _node_name = None

    def set_node_name(self, node_name: str) -> None:
        self._node_name = node_name



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddActionTypeTrait import AddActionTypeTrait
from pohoda.entity.common.AddParameterToHeaderTrait import AddParameterToHeaderTrait
from pohoda.entity.order.Header import Header
from pohoda.entity.order.Item import Item
from pohoda.entity.order.Summary import Summary


class Order(Agenda, AddActionTypeTrait, AddParameterToHeaderTrait):
    import_root = 'lst:order'

    def __init__(self, data: dict, ico: str):
        if data:
            data = {'header': Header(data, ico)}

        super().__init__(data, ico)

    def add_item(self, data: dict) -> 'Order':
        """
        Add order item.
        :param data:
        :return:
        """

        if not self._data.get('orderDetail'):
            self._data['orderDetail'] = []

        self._data['orderDetail'].append(Item(data, self._ico))
        return self

    def add_summary(self, data: dict) -> 'Order':
        """
        Add order summary.
        :param data:
        :return:
        """

        self._data['summary'] = Summary(data, self._ico)
        return self

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('order', namespace='ord')
        xml.set('version', '2.0')
        self._add_elements(xml, ['actionType', 'header', 'orderDetail', 'summary'], 'ord')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.print_request.Pdf import Pdf
from pohoda.entity.print_request.Report import Report


class PrinterSettings(Agenda):
    _elements = ['report', 'printer', 'pdf']

    def __init__(self, data: dict, ico: str):
        # process report
        report = data.get('report')
        if report:
            data['report'] = Report(report, ico)

        # process pdf
        pdf = data.get('pdf')
        if pdf:
            data['pdf'] = Pdf(pdf, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('printerSettings', namespace='prn')
        self._add_elements(xml, self._elements, 'prn')
        return xml



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.print_request.Filter import Filter


class Record(Agenda):
    def __init__(self, data: dict, ico: str):
        # process filter
        filter_ = data.get('filter')
        if filter_:
            data['filter'] = Filter(filter_, ico)
        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('record', 'prn')
        xml.set('agenda', self._data['agenda'])
        self._add_elements(xml, ['filter'], 'prn')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.invoice.Item import Item


class AdvancePaymentItem(Item):
    _ref_elements = ['sourceDocument', 'accounting', 'classificationVAT', 'classificationKVDPH', 'centre', 'activity',
                     'contract']
    _elements = ['sourceDocument', 'quantity', 'payVAT', 'rateVAT', 'discountPercentage', 'homeCurrency',
                 'foreignCurrency', 'note', 'accounting', 'classificationVAT', 'classificationKVDPH', 'centre',
                 'activity', 'contract']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('invoiceAdvancePaymentItem', namespace='inv')
        self._add_elements(xml, self._elements + ['parameters'], 'inv')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterToHeaderTrait import AddParameterToHeaderTrait
from pohoda.entity.invoice.AdvancePaymentItem import AdvancePaymentItem
from pohoda.entity.invoice.Header import Header
from pohoda.entity.invoice.Item import Item
from pohoda.entity.invoice.Summary import Summary
from pohoda.entity.type.Link import Link


class Invoice(Agenda, AddParameterToHeaderTrait):
    import_root = 'lst:invoice'

    def __init__(self, data: dict, ico: str):

        if 'invoiceType' not in data:
            data['invoiceType'] = 'issuedInvoice'

        data = {'header': Header(data, ico)}
        super().__init__(data, ico)

    def add_link(self, data: dict) -> 'Invoice':
        """
        Add link.
        :param data:
        :return:
        """
        if not self._data.get('links'):
            self._data['links'] = []

        self._data['links'].append(Link(data, self._ico))

        return self

    def add_item(self, data: dict) -> 'Invoice':
        """
        Add invoice item.
        :param data:
        :return:
        """

        if not self._data.get('invoiceDetail'):
            self._data['invoiceDetail'] = []

        self._data['invoiceDetail'].append(Item(data, self._ico))
        return self

    def add_advance_payment_item(self, data: dict) -> 'Invoice':
        """
        Add advance payment item.
        :param data:
        :return:
        """

        if not self._data.get('invoiceDetail'):
            self._data['invoiceDetail'] = []

        self._data['invoiceDetail'].append(AdvancePaymentItem(data, self._ico))
        return self

    def add_summary(self, data: dict) -> 'Invoice':
        """
        Add invoice summary.
        :param data:
        :return:
        """

        self._data['summary'] = Summary(data, self._ico)
        return self

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('invoice', namespace='inv')
        xml.set('version', '2.0')
        self._add_elements(xml, ['links', 'header', 'invoiceDetail', 'summary'], 'inv')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda


class ItemUserCode(Agenda):

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('itemUserCode', namespace='lst')
        xml.set('code', self._data['code'])
        xml.set('name', self._data['name'])
        if self._data.get('constants'):
            xml.set('constants', self._data['constants'])

        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda


class AddressInternetType(Agenda):
    _elements = ['company', 'title', 'surname', 'name', 'city', 'street', 'number', 'zip', 'ico', 'dic', 'icDph',
                 'phone', 'mobilPhone', 'fax', 'email', 'www']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('address', namespace='typ')
        self._add_elements(xml, self._elements, 'typ')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.type.Address import Address


class Header(Agenda, AddParameterTrait):
    _ref_elements = ['centre', 'activity', 'contract', 'number']

    _elements = ['identity', 'region', 'phone', 'mobil', 'fax', 'email', 'web', 'ICQ', 'Skype', 'GPS', 'credit',
                 'priceIDS', 'maturity', 'paymentType', 'agreement', 'number', 'ost1', 'ost2', 'p1', 'p2', 'p3', 'p4',
                 'p5', 'p6', 'markRecord', 'message', 'note', 'intNote', 'centre', 'activity', 'contract', 'adGroup']

    def __init__(self, data: dict, ico: str):
        identity = data.get('identity')
        if identity:
            data['identity'] = Address(identity, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('addressbookHeader', namespace='adb')
        self._add_elements(xml, self._elements + ['parameters'], 'adb')
        return xml



import html
import datetime
from typing import Dict, List, Union, Any, Optional, Tuple
from lxml import etree
from pohoda.entity.common.SetNamespaceTrait import SetNamespaceTrait
from pohoda.entity.common.SetNodeNameTrait import SetNodeNameTrait


class Agenda:
    namespaces = {
        'adb': 'http://www.stormware.cz/schema/version_2/addressbook.xsd',
        'con': 'http://www.stormware.cz/schema/version_2/contract.xsd',
        'ctg': 'http://www.stormware.cz/schema/version_2/category.xsd',
        'dat': 'http://www.stormware.cz/schema/version_2/data.xsd',
        'ftr': 'http://www.stormware.cz/schema/version_2/filter.xsd',
        'int': 'http://www.stormware.cz/schema/version_2/intDoc.xsd',
        'inv': 'http://www.stormware.cz/schema/version_2/invoice.xsd',
        'ipm': 'http://www.stormware.cz/schema/version_2/intParam.xsd',
        'lAdb': 'http://www.stormware.cz/schema/version_2/list_addBook.xsd',
        'lst': 'http://www.stormware.cz/schema/version_2/list.xsd',
        'lStk': 'http://www.stormware.cz/schema/version_2/list_stock.xsd',
        'ord': 'http://www.stormware.cz/schema/version_2/order.xsd',
        'pre': 'http://www.stormware.cz/schema/version_2/prevodka.xsd',
        'pri': 'http://www.stormware.cz/schema/version_2/prijemka.xsd',
        'prn': 'http://www.stormware.cz/schema/version_2/print.xsd',
        'pro': 'http://www.stormware.cz/schema/version_2/prodejka.xsd',
        'str': 'http://www.stormware.cz/schema/version_2/storage.xsd',
        'stk': 'http://www.stormware.cz/schema/version_2/stock.xsd',
        'typ': 'http://www.stormware.cz/schema/version_2/type.xsd',
        'vyd': 'http://www.stormware.cz/schema/version_2/vydejka.xsd',
        'ofr': 'http://www.stormware.cz/schema/version_2/offer.xsd'
    }

    _data = {}  # type: Dict[str, Any]

    _ref_elements = []  # type: List[str]

    _elements_attributes_mapper = {}  # type: Dict[str, Tuple[str, str, Optional[str]]]

    def __init__(self, data: dict, ico: str):
        """
        Construct agenda using provided data.
        :param data:
        :param ico:
        """

        self._ico = ico
        self._data = data

    def get_xml(self) -> etree.Element:
        """
        Get XML.
        :return:
        """
        raise NotImplementedError

    def _create_xml_tag(self, tag: str, namespace: Optional[str] = None) -> etree.Element:
        """
        Create XML.
        :return:
        """

        return etree.Element(self.with_xml_namespace(namespace, tag)) if namespace else etree.Element(tag)

    @staticmethod
    def _element_data_to_xml(element_data: Any) -> str:
        # Object is datetime (extends date)
        if isinstance(element_data, datetime.datetime):
            return html.escape(element_data.strftime('%Y-%m-%dT%H:%M:%S'))
        # Object is date
        if isinstance(element_data, datetime.date):
            return html.escape(element_data.strftime('%Y-%m-%d'))
        # Object is time
        if isinstance(element_data, datetime.time):
            return html.escape(element_data.strftime('%H:%M:%S'))

        if isinstance(element_data, bool):
            return 'true' if element_data else 'false'

        return html.escape(str(element_data))

    @staticmethod
    def with_xml_namespace(namespace_key: str, tag_name: str) -> str:
        found_namespace = Agenda.namespaces.get(namespace_key)
        if not found_namespace:
            ValueError('Unknown namespace key {}'.format(namespace_key))
        return '{{{}}}{}'.format(found_namespace, tag_name)

    def _add_elements(self, xml: etree.Element, elements: List[str], namespace: Optional[str] = None) -> None:
        """
        Add batch elements.
        :param xml:
        :param elements:
        :param namespace:
        :return:
        """
        for element in elements:
            found_element_in_data = self._data.get(element)
            if found_element_in_data is None:
                continue

            # is ref element
            if element in self._ref_elements:
                self._add_ref_element(xml, element, found_element_in_data, namespace)
                continue

            found_element_attributes_mapper = self._elements_attributes_mapper.get(element)
            if found_element_attributes_mapper:
                attr_element, attr_name, attr_namespace = found_element_attributes_mapper

                # get element
                element_name = self.with_xml_namespace(namespace, attr_element) if namespace else attr_element
                attr_element_found = xml.find(element_name)

                if attr_element_found is not None:
                    attr_element_name = self.with_xml_namespace(attr_namespace, attr_name) if attr_namespace else attr_name
                    attr_element_found.set(attr_element_name, self._element_data_to_xml(found_element_in_data))

                continue

            # Agenda object
            if isinstance(found_element_in_data, Agenda):

                # set namespace
                if namespace and isinstance(found_element_in_data, SetNamespaceTrait):
                    found_element_in_data.set_namespace(namespace)

                # set node name
                if isinstance(found_element_in_data, SetNodeNameTrait):
                    found_element_in_data.set_node_name(element)

                xml.append(found_element_in_data.get_xml())
                continue

            child = etree.Element(self.with_xml_namespace(namespace, element) if namespace else element)

            # list of Agenda objects
            if isinstance(found_element_in_data, list):
                for node in found_element_in_data:
                    child.append(node.get_xml())
            else:
                # Anything else
                child.text = self._element_data_to_xml(found_element_in_data)
            xml.append(child)

    def _add_ref_element(self,
                         xml: etree.Element,
                         name: str,
                         value: Union[dict, str],
                         namespace: Optional[str] = None
                         ) -> etree.Element:
        """
        Add ref element.
        :param xml:
        :param name:
        :param value:
        :param namespace:
        :return:
        """

        node = etree.Element(self.with_xml_namespace(namespace, name) if namespace else name)

        if not isinstance(value, dict):
            value = {'ids': value}

        for k, v in value.items():
            node_child = etree.Element(self.with_xml_namespace('typ', k))
            node_child.text = html.escape(str(v))
            node.append(node_child)

        xml.append(node)
        return node



# coding: utf-8
from lxml import etree
from pohoda.entity.document.Part import Part
from pohoda.entity.type.CurrencyForeign import CurrencyForeign
from pohoda.entity.type.CurrencyHome import CurrencyHome


class Summary(Part):

    def __init__(self, data: dict, ico: str):
        # process home currency
        home_currency = data.get('homeCurrency')
        if home_currency:
            data['homeCurrency'] = CurrencyHome(home_currency, ico)

        # process foreign currency
        foreign_currency = data.get('foreignCurrency')
        if foreign_currency:
            data['foreignCurrency'] = CurrencyForeign(foreign_currency, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        if not self._namespace:
            raise ValueError('Namespace not set.')

        if not self._node_prefix:
            raise ValueError('Node name prefix not set.')

        xml = self._create_xml_tag('{}Summary'.format(self._node_prefix), namespace=self._namespace)
        self._add_elements(xml, self._elements, self._namespace)
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.type.Address import Address
from pohoda.entity.type.MyAddress import MyAddress


class Header(Agenda, AddParameterTrait):
    _ref_elements = ['number', 'accounting', 'classificationVAT', 'classificationKVDPH', 'centre', 'activity',
                     'contract', 'regVATinEU', 'MOSS', 'evidentiaryResourcesMOSS']
    _elements = ['number', 'symVar', 'symPar', 'originalDocumentNumber', 'originalCorrectiveDocument', 'date',
                 'dateTax', 'dateAccounting', 'dateDelivery', 'dateKVDPH', 'dateKHDPH', 'accounting',
                 'classificationVAT', 'classificationKVDPH', 'numberKHDPH', 'text', 'partnerIdentity', 'myIdentity',
                 'liquidation', 'centre', 'activity', 'contract', 'regVATinEU', 'MOSS', 'evidentiaryResourcesMOSS',
                 'accountingPeriodMOSS', 'note', 'intNote', 'markRecord']

    def __init__(self, data: dict, ico: str):
        # process partner identity
        partner_identity = data.get('partnerIdentity')
        if partner_identity:
            data['partnerIdentity'] = Address(partner_identity, ico)

        # process my identity
        my_identity = data.get('myIdentity')
        if my_identity:
            data['myIdentity'] = MyAddress(my_identity, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('intDocHeader', namespace='int')
        self._add_elements(xml, self._elements + ['parameters'], 'int')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda


class Link(Agenda):
    _ref_elements = ['sourceDocument', 'settingsSourceDocument']
    _elements = ['sourceAgenda', 'sourceDocument', 'settingsSourceDocument']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('link', 'typ')
        self._add_elements(xml, self._elements, 'typ')
        return xml



# coding: utf-8
from typing import Any
from pohoda.entity.type.Parameter import Parameter
from pohoda.entity.common.Trait import Trait


class AddParameterTrait(Trait):

    def add_parameter(self, name: str, parameter_type: str, value: Any, list_: Any = None) -> 'AddParameterTrait':
        """
        Set user-defined parameter.
        :param name: (can be set without preceding VPr / RefVPr)
        :param parameter_type:
        :param value:
        :param list_:
        :return:
        """

        if not self._data.get('parameters'):
            self._data['parameters'] = []

        self._data['parameters'].append(Parameter({
            'name': name,
            'type': parameter_type,
            'value': value,
            'list': list_
        }, self._ico))

        return self



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda


class Filter(Agenda):
    _elements = ['id']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('filter', namespace='ftr')
        self._add_elements(xml, self._elements, 'ftr')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda


class Storage(Agenda):
    import_root = 'lst:itemStorage'

    def add_substorage(self, storage: 'Storage') -> None:
        """
        Add substorage.
        :param storage:
        :return:
        """

        if not self._data.get('subStorages'):
            self._data['subStorages'] = []

        self._data['subStorages'].append(storage)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('storage', namespace='str')
        xml.set('version', '2.0')
        self.storage_xml(xml)
        return xml

    def storage_xml(self, xml: etree.Element) -> None:
        """
        Attach storage to XML element.
        :param xml:
        :return:
        """

        storage = self._create_xml_tag('itemStorage', namespace='str')
        storage.set('code', self._data['code'])
        xml.append(storage)

        if self._data.get('name'):
            storage.set('name', self._data['name'])

        sub_storages = self._data.get('subStorages')
        if sub_storages:
            sub_storages_element = self._create_xml_tag('subStorages', namespace='str')
            storage.append(sub_storages_element)
            for sub_storage in sub_storages:
                sub_storage.storage_xml(sub_storages_element)



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.SetNamespaceTrait import SetNamespaceTrait
from pohoda.entity.common.SetNodeNameTrait import SetNodeNameTrait


class CurrencyForeign(Agenda, SetNamespaceTrait, SetNodeNameTrait):
    _ref_elements = ['currency']
    _elements = ['currency', 'rate', 'amount', 'priceSum']

    def get_xml(self) -> etree.Element:

        if not self._namespace:
            raise ValueError('Namespace not set.')

        if not self._node_name:
            raise ValueError('Node name not set.')

        xml = self._create_xml_tag(self._node_name, namespace=self._namespace)
        self._add_elements(xml, self._elements, 'typ')
        return xml



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class Picture(Agenda):

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('picture', namespace='stk')
        xml.set('default', 'true' if self._data['default'] else 'false')
        self._add_elements(xml, ['filepath', 'description', 'order'], 'stk')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.user_list.ItemUserCode import ItemUserCode


class UserList(Agenda):
    import_root = 'lst:listUserCode'

    def add_item_user_code(self, data: dict) -> None:
        """
        Add item user code.
        :param data:
        :return:
        """

        if not self._data.get('itemUserCodes'):
            self._data['itemUserCodes'] = []

        self._data['itemUserCodes'].append(ItemUserCode(data, self._ico))

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('listUserCode', namespace='lst')
        xml.set('version', '1.1')
        xml.set('code', self._data['code'])
        xml.set('name', self._data['name'])

        constants = self._data.get('constants', 'false')
        if constants == 'true':
            xml.set('constants', constants)

        item_user_codes = self._data.get('itemUserCodes')
        if item_user_codes:
            for item_user_code in item_user_codes:
                xml.append(item_user_code.get_xml())

        return xml



# coding: utf-8
from typing import Any
from pohoda.entity.common.Trait import Trait


class AddParameterToHeaderTrait(Trait):
    def add_parameter(self,
                      name: str,
                      parameter_type: str,
                      value: Any,
                      list_: Any = None) -> 'AddParameterToHeaderTrait':
        """
        Set user-defined parameter.
        :param name: (can be set without preceding VPr / RefVPr)
        :param parameter_type:
        :param value:
        :param list_:
        :return:
        """

        self._data['header'].add_parameter(name, parameter_type, value, list_)
        return self



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class ShipToAddressType(Agenda):
    _ref_elements = ['country']
    _elements = ['company', 'division', 'name', 'city', 'street', 'zip', 'country', 'phone', 'email',
                 'defaultShipAddress']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('shipToAddress', namespace='typ')
        self._add_elements(xml, self._elements, 'typ')
        return xml



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class IntParameter(Agenda):

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('intParameter', namespace='stk')
        self._add_elements(xml, ['intParameterID', 'intParameterType'], 'stk')

        values = self._create_xml_tag('intParameterValues', namespace='stk')
        xml.appen(values)

        int_value = self._create_xml_tag('intParameterValue', namespace='stk')
        values.append(int_value)

        value = self._create_xml_tag('parameterValue', namespace='stk')
        value.text = self._data['value']
        int_value.append(value)

        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda


class Category(Agenda):
    def get_xml(self) -> etree.Element:
        element = self._create_xml_tag('idCategory', namespace='stk')
        element.text = str(self._data['idCategory'])
        return element



import warnings
from typing import Any


class Trait:
    def __getattr__(self, name: str) -> Any:
        """ will only get called for undefined attributes """
        warnings.warn('No member "{}" contained in trait.'.format(name))



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.type.Address import Address
from pohoda.entity.type.MyAddress import MyAddress


class Header(Agenda, AddParameterTrait):
    _ref_elements = ['extId', 'number', 'accounting', 'classificationVAT', 'classificationKVDPH', 'order', 'paymentType',
                     'priceLevel', 'account', 'paymentAccount', 'centre', 'activity', 'contract', 'regVATinEU', 'MOSS',
                     'evidentiaryResourcesMOSS', 'carrier']
    _elements = ['extId', 'invoiceType', 'number', 'symVar', 'originalDocument', 'originalDocumentNumber', 'symPar',
                 'date', 'dateTax', 'dateAccounting', 'dateKHDPH', 'dateDue', 'dateApplicationVAT', 'dateDelivery',
                 'accounting', 'classificationVAT', 'classificationKVDPH', 'numberKHDPH', 'text', 'partnerIdentity',
                 'myIdentity', 'order', 'numberOrder', 'dateOrder', 'paymentType', 'priceLevel', 'account', 'symConst',
                 'symSpec', 'paymentAccount', 'paymentTerminal', 'centre', 'activity', 'contract', 'regVATinEU', 'MOSS',
                 'evidentiaryResourcesMOSS', 'accountingPeriodMOSS', 'note', 'carrier', 'intNote']

    def __init__(self, data: dict, ico: str):
        partner_identity = data.get('partnerIdentity')
        if partner_identity:
            data['partnerIdentity'] = Address(partner_identity, ico)

        my_identity = data.get('myIdentity')
        if my_identity:
            data['myIdentity'] = MyAddress(my_identity, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml_ = self._create_xml_tag('invoiceHeader', namespace='inv')
        self._add_elements(xml_, self._elements + ['parameters'], 'inv')
        return xml_



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.int_param.Settings import Settings


class IntParam(Agenda):
    import_root = 'lst:intParamDetail'
    _elements = ['name', 'description', 'parameterType', 'parameterSettings']

    def __init__(self, data: dict, ico: str):
        # prepare empty parameter list for list
        if data['parameterType'] == 'listValue':
            data['parameterSettings'] = {'parameterList': []}

        # process settings
        parameter_settings = data.get('parameterSettings')
        if parameter_settings:
            data['parameterSettings'] = Settings(parameter_settings, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('intParamDetail', namespace='ipm')
        xml.set('version', '2.0')
        param = xml.addchild('ipm:intParam')
        self._add_elements(param, self._elements, 'ipm')
        return xml



from typing import List
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.SetNamespaceTrait import SetNamespaceTrait


class Part(Agenda, SetNamespaceTrait):
    _node_prefix = None

    _elements = []  # type: List[str]

    def set_node_prefix(self, prefix: str) -> None:
        self._node_prefix = prefix

    def get_xml(self) -> etree.Element:
        raise NotImplementedError



# coding: utf-8
from typing import Optional
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddActionTypeTrait import AddActionTypeTrait
from pohoda.entity.common.AddParameterToHeaderTrait import AddParameterToHeaderTrait
from pohoda.entity.stock.Header import Header
from pohoda.entity.stock.Price import Price


class Stock(Agenda, AddActionTypeTrait, AddParameterToHeaderTrait):
    import_root = 'lStk:stock'

    def __init__(self, data: dict, ico: str):
        if data:
            data = {'header': Header(data, ico)}
        # end if
        super().__init__(data, ico)

    def add_price(self, code: str, value: float) -> 'Stock':
        """
        Add price.
        :param code:
        :param value:
        :return:
        """

        if not self._data.get('stockPriceItem'):
            self._data['stockPriceItem'] = []

        self._data['stockPriceItem'].append(Price({'code': code, 'value': value}, self._ico))
        return self

    def add_image(self,
                  filepath: str,
                  description: Optional[str] = None,
                  order: Optional[int] = None,
                  default: bool = False
                  ) -> 'Stock':
        """
        Add image.
        :param filepath:
        :param description:
        :param order:
        :param default:
        :return:
        """
        self._data['header'].add_image(filepath, description, order, default)
        return self

    def add_category(self, category_id: int) -> 'Stock':
        """
        Add category.
        :param category_id:
        :return:
        """

        self._data['header'].add_category(category_id)
        return self

    def add_int_parameter(self, data: dict) -> 'Stock':
        """
        Add int parameter.
        :param data:
        :return:
        """

        self._data['header'].add_int_parameter(data)
        return self

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('stock', namespace='stk')
        xml.set('version', '2.0')
        self._add_elements(xml, ['actionType', 'header', 'stockPriceItem'], 'stk')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.type.Address import Address
from pohoda.entity.type.MyAddress import MyAddress


class Header(Agenda, AddParameterTrait):
    _ref_elements = ['number', 'priceLevel', 'centre', 'activity', 'contract', 'regVATinEU', 'MOSS', 'evidentiaryResourcesMOSS']
    _elements = ['offerType', 'number', 'date', 'validTill', 'text', 'partnerIdentity', 'myIdentity', 'priceLevel', 'centre',
                 'activity', 'contract', 'regVATinEU', 'MOSS', 'evidentiaryResourcesMOSS', 'accountingPeriodMOSS', 'isExecuted',
                 'details', 'note', 'intNote', 'markRecord']

    def __init__(self, data: dict, ico: str):
        # process partner identity
        partner_identity = data.get('partnerIdentity')
        if partner_identity:
            data['partnerIdentity'] = Address(partner_identity, ico)

        # process my identity
        my_identity = data.get('myIdentity')
        if my_identity:
            data['myIdentity'] = MyAddress(my_identity, ico)

        if 'offerType' not in data:
            data['offerType'] = 'receivedOffer'

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('offerHeader', namespace='ord')
        self._add_elements(xml, self._elements + ['parameters'], 'ord')
        return xml



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class UserFilterName(Agenda):
    def get_xml(self) -> etree.Element:
        element = self._create_xml_tag('userFilterName', namespace='ftr')
        element.text = self._data['userFilterName']
        return element



# coding: utf-8

from typing import Optional, Any
from pohoda.entity.type.ActionType import ActionType
from pohoda.entity.common.Trait import Trait


class AddActionTypeTrait(Trait):

    def add_action_type(self,
                        type_: Optional[str] = None,
                        filter_: Optional[Any] = None,
                        agenda: Optional[str] = None
                        ) -> 'AddActionTypeTrait':
        """
        Add action type.
        :param type_:
        :param filter_:
        :param agenda:
        :return:
        """

        action_type = self._data.get('actionType')
        if action_type:
            raise ValueError('Duplicate action type.')

        self._data['actionType'] = ActionType({'type': type_, 'filter': filter_, 'agenda': agenda}, self._ico)
        return self



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.list_request.Filter import Filter
from pohoda.entity.list_request.UserFilterName import UserFilterName


class ListRequest(Agenda):

    def __init__(self, data: dict, ico: str):

        if 'type' not in data:
            raise ValueError('"type" argument is required')

        data['type'] = {
            'Addressbook': 'AddressBook',
            'IssueSlip': 'Vydejka'
        }.get(data['type'], data['type'])

        if 'namespace' not in data:
            data['namespace'] = {
                'Stock': 'lStk',
                'AddressBook': 'lAdb'
            }.get(data['type'], 'lst')

        if 'orderType' not in data:
            data['orderType'] = {
                'Order': 'receivedOrder'
            }.get(data['type'])

        if 'invoiceType' not in data:
            data['invoiceType'] = {
                'Invoice': 'issuedInvoice'
            }.get(data['type'])

        super().__init__(data, ico)

    def add_filter(self, data: dict) -> 'ListRequest':
        """
        Add filter.
        :param data:
        :return:
        """

        self._data['filter'] = Filter(data, self._ico)
        return self

    def add_user_filter_name(self, name: str) -> 'ListRequest':
        """
        Add user filter name.
        :param name:
        :return:
        """

        self._data['userFilterName'] = UserFilterName({'userFilterName': name}, self._ico)
        return self

    def get_xml(self) -> etree.Element:
        # UserList is custom
        if self._data['type'] == 'UserList':
            # Custom export for UserList
            xml = self._create_xml_tag(
                'listUserCodeRequest',
                namespace=self._data['namespace']
            )
            xml.set('version', '1.1')
            xml.set('listVersion', '1.1')
            return xml

        xml = self._create_xml_tag(
            'list{}Request'.format(self._data['type']),
            namespace=self._data['namespace']
        )
        xml.set('version', '2.0')

        # IntParam doesn't have the version attribute
        if self._data['type'] != 'IntParam':
            xml.set(self._get_lc_first_type() + 'Version', '2.0')

        first_type = self._data.get(self._get_lc_first_type() + 'Type')
        if first_type:
            xml.set(self._get_lc_first_type() + 'Type', first_type)

        request = self._create_xml_tag('request{}'.format(self._data['type']), namespace=self._data['namespace'])
        xml.append(request)
        self._add_elements(request, ['filter', 'userFilterName'], 'ftr')
        return xml

    def _get_lc_first_type(self) -> str:
        """
        Get LC first type name.
        :return:
        """

        # ActionPrice is custom
        if self._data['type'] == 'ActionPrice':
            return 'actionPrices'

        modify_str = self._data['type']
        return modify_str[0].lower() + modify_str[1:]



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class Intrastat(Agenda):
    _elements = ['goodsCode', 'description', 'statistic', 'unit', 'coefficient', 'country']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('intrastat', namespace='stk')
        self._add_elements(xml, self._elements, 'stk')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterToHeaderTrait import AddParameterToHeaderTrait
from pohoda.entity.stock_transfer.Header import Header
from pohoda.entity.stock_transfer.Item import Item


class StockTransfer(Agenda, AddParameterToHeaderTrait):
    import_root = 'lst:prevodka'

    def __init__(self, data: dict, ico: str):
        data = {'header': Header(data, ico)}
        super().__init__(data, ico)

    def add_item(self, data: dict) -> 'StockTransfer':
        """
        Add item.
        :param data:
        :return:
        """

        if not self._data.get('prevodkaDetail'):
            self._data['prevodkaDetail'] = []

        self._data['prevodkaDetail'].append(Item(data, self._ico))
        return self

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('prevodka', namespace='pre')
        xml.set('version', '2.0')
        self._add_elements(xml, ['header', 'prevodkaDetail'], 'pre')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterToHeaderTrait import AddParameterToHeaderTrait
from pohoda.entity.issue_slip.Header import Header
from pohoda.entity.issue_slip.Item import Item
from pohoda.entity.issue_slip.Summary import Summary
from pohoda.entity.type.Link import Link


class IssueSlip(Agenda, AddParameterToHeaderTrait):
    import_root = 'lst:vydejka'

    def __init__(self, data: dict, ico: str):
        data = {'header': Header(data, ico)}
        super().__init__(data, ico)

    def add_item(self, data: dict) -> 'IssueSlip':
        """
        Add item.
        :param data:
        :return:
        """

        if not self._data.get('vydejkaDetail'):
            self._data['vydejkaDetail'] = []

        self._data['vydejkaDetail'].append(Item(data, self._ico))
        return self

    def add_summary(self, data: dict) -> 'IssueSlip':
        """
        Add summary.
        :param data:
        :return:
        """

        self._data['summary'] = Summary(data, self._ico)
        return self

    def add_link(self, data: dict) -> 'IssueSlip':
        links = self._data.get('links', [])
        links.append(Link(data, self._ico))
        self._data['links'] = links
        return self

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('vydejka', namespace='vyd')
        xml.set('version', '2.0')
        self._add_elements(xml, ['links', 'header', 'vydejkaDetail', 'summary'], 'vyd')
        return xml



# coding: utf-8
from typing import Optional
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.stock.Category import Category
from pohoda.entity.stock.IntParameter import IntParameter
from pohoda.entity.stock.Picture import Picture
from pohoda.entity.stock.Intrastat import Intrastat
from pohoda.entity.stock.RecyclingContrib import RecyclingContrib


class Header(Agenda, AddParameterTrait):
    _ref_elements = ['storage', 'typePrice', 'typeRP', 'supplier']
    _elements_attributes_mapper = {
        'purchasingPricePayVAT': ('purchasingPrice', 'payVAT', None),
        'sellingPricePayVAT': ('sellingPrice', 'payVAT', None)
    }
    _elements = ['stockType', 'code', 'EAN', 'PLU', 'isSales', 'isSerialNumber', 'isInternet', 'isBatch',
                 'purchasingRateVAT', 'sellingRateVAT', 'name', 'nameComplement', 'unit', 'unit2', 'unit3',
                 'coefficient2', 'coefficient3', 'storage', 'typePrice', 'purchasingPrice', 'purchasingPricePayVAT',
                 'sellingPrice', 'sellingPricePayVAT', 'limitMin', 'limitMax', 'mass', 'volume', 'supplier',
                 'orderName', 'orderQuantity', 'shortName', 'typeRP', 'guaranteeType', 'guarantee', 'producer',
                 'description', 'description2', 'note', 'intrastat', 'recyclingContrib']
    _images_counter = 0

    def __init__(self, data: dict, ico: str):
        if 'stockType' not in data:
            data['stockType'] = 'card'

        if 'intrastat' in data:
            data['intrastat'] = Intrastat(data['intrastat'], ico)

        if 'recyclingContrib' in data:
            data['recyclingContrib'] = RecyclingContrib(data['recyclingContrib'], ico)

        super().__init__(data, ico)

    def add_image(self,
                  filepath: str,
                  description: Optional[str] = None,
                  order: Optional[int] = None,
                  default: bool = False
                  ) -> None:
        """
        Add image.
        :param filepath:
        :param description:
        :param order:
        :param default:
        :return:
        """
        if not self._data.get('pictures'):
            self._data['pictures'] = []

        if not order:
            self._images_counter += 1
            order = self._images_counter

        self._data['pictures'].append(Picture(
            {
                'filepath': filepath,
                'description': description,
                'order': order,
                'default': default
            },
            self._ico
        ))

    def add_category(self, category_id: int) -> None:
        """
        Add category.
        :param category_id:
        :return:
        """
        if not self._data.get('categories'):
            self._data['categories'] = []

        self._data['categories'].append(Category(
            {
                'idCategory': category_id
            },
            self._ico
        ))

    def add_int_parameter(self, data: dict) -> None:
        """
        Add int parameter.
        :param data:
        :return:
        """

        if not self._data.get('intParameters'):
            self._data['intParameters'] = []

        self._data['intParameters'].append(IntParameter(data, self._ico))

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('stockHeader', namespace='stk')
        self._add_elements(xml, self._elements + ['categories', 'pictures', 'parameters', 'intParameters'], 'stk')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.SetNamespaceTrait import SetNamespaceTrait
from pohoda.entity.common.SetNodeNameTrait import SetNodeNameTrait
from pohoda.entity.type.AddressType import AddressType
from pohoda.entity.type.ShipToAddressType import ShipToAddressType


class Address(Agenda, SetNamespaceTrait, SetNodeNameTrait):
    _ref_elements = ['extId']
    _elements = ['id', 'extId', 'address', 'addressLinkToAddress', 'shipToAddress']

    _elements_attributes_mapper = {'addressLinkToAddress': ('address', 'linkToAddress', None)}

    def __init__(self, data: dict, ico: str):
        address = data.get('address')
        if address:
            data['address'] = AddressType(address, ico)

        ship_to_address = data.get('shipToAddress')
        if ship_to_address:
            data['shipToAddress'] = ShipToAddressType(data['shipToAddress'], ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:

        if not self._namespace:
            raise ValueError('Namespace not set.')

        if not self._node_name:
            raise ValueError('Node name not set.')

        xml = self._create_xml_tag(self._node_name, namespace=self._namespace)
        self._add_elements(xml, self._elements, 'typ')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.SetNamespaceTrait import SetNamespaceTrait


class ActionType(Agenda, SetNamespaceTrait):
    def get_xml(self) -> etree.Element:

        if not self._namespace:
            raise ValueError('Namespace not set.')
        # end if
        xml = self._create_xml_tag('actionType', namespace=self._namespace)
        action = self._create_xml_tag('add' if self._data['type'] == 'add/update' else self._data['type'],
                                      namespace=self._namespace)
        xml.append(action)
        if self._data['type'] == 'add/update':
            action.set('update', 'true')

        if self._data['filter']:

            filter_ = self._create_xml_tag('filter', namespace='ftr')
            action.append(filter_)
            if self._data['agenda']:
                filter_.set('agenda', self._data['agenda'])

            for property_, value_ in self._data['filter'].items():
                ftr = self._create_xml_tag(property_, namespace='ftr')
                filter_.append(ftr)
                ftr.text = None if isinstance(value_, dict) else value_
                if isinstance(value_, dict):
                    for t_property, t_value in value_.items():
                        ftr_child = self._create_xml_tag(t_property, namespace='typ')
                        ftr_child.text = t_value
                        ftr.append(ftr_child)
        return xml



import re
from typing import Any, Optional
from importlib import import_module
import humps
from lxml import etree

from pohoda.entity.Agenda import Agenda


class Pohoda:
    _encoding = 'windows-1250'
    _ico = None
    _application = 'Python Pohoda connector'

    _xml_root = None
    _filename = None

    def __init__(self, ico: str, id_: str, note: Optional[str] = None, encoding: str = 'windows-1250'):
        self._encoding = encoding
        self._ico = ico

        # Register namespaces
        for namespace_key, namespace_value in Agenda.namespaces.items():
            etree.register_namespace(namespace_key, namespace_value)

        self.initialize_xml_root(id_, note)

    def set_application_name(self, name: str) -> None:
        """
        Set the name of the application.
        :param name:
        :return:
        """
        self._application = name

    def create(self, name: str, data: Optional[dict] = None) -> Agenda:
        """
        Create and return instance of requested agenda.
        :param name:
        :param data:
        :return: Agenda
        """
        if data is None:
            data = {}

        try:
            entity = getattr(import_module('pohoda.entity.{}'.format(name)), name)
            if not issubclass(entity, Agenda):
                raise ValueError('Not allowed entity: {}'.format(name))
            return entity(data, self._ico)
        except ImportError as e:
            raise ValueError('Entity {} not found'.format(name)) from e

    def initialize_xml_root(self, id_: str, note: Optional[str] = None) -> None:
        """
        Open new XML file for writing.
        :param id_:
        :param note:
        :return:
        """

        if not note:
            note = ''

        self._xml_root = etree.Element(Agenda.with_xml_namespace('dat', 'dataPack'))
        self._xml_root.set('ico', self._ico)
        self._xml_root.set('application', self._application)
        self._xml_root.set('version', '2.0')

        if note:
            self._xml_root.set('note', note)

        if id_:
            self._xml_root.set('id', id_)

    def add_item(self, id_: str, agenda: Agenda) -> None:
        """
        Add item.
        :param id_:
        :param agenda:
        :return:
        """
        data_pack_item = etree.SubElement(self._xml_root, Agenda.with_xml_namespace('dat', 'dataPackItem'))
        data_pack_item.set('id', id_)
        data_pack_item.set('version', '2.0')
        data_pack_item.append(agenda.get_xml())

    def write(self, filename: str, pretty_print: bool = True) -> None:
        """
        Write xml tree to file
        :param filename:
        :param pretty_print:
        :return:
        """

        tree = etree.ElementTree(self._xml_root)
        tree.write(filename, xml_declaration=True, encoding=self._encoding, method='xml', pretty_print=pretty_print)

    def load(self, name: str, filename: str) -> None:
        raise NotImplementedError

    def next(self) -> None:
        raise NotImplementedError

    def __getattr__(self, name: str) -> Any:
        """
        Handle dynamic method calls.
        :param name:
        :return:
        """

        def _getattr_resolver(*args: Any) -> Any:
            match_create = re.match(r'^create_([a-zA-Z0-9_]*)$', name)
            if match_create:
                return getattr(self, 'create')(humps.pascalize(match_create.group(1)), args[0])

            match_load = re.match(r'^load_([a-zA-Z0-9_]*)$', name)
            if match_load:
                if not args[0]:
                    raise ValueError('Filename is not set')
                return getattr(self, 'load')(humps.pascalize(match_load.group(1)), args[0])

            raise ValueError('Unknown method: {}'.format(name))

        return _getattr_resolver



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.type.CurrencyItem import CurrencyItem
from pohoda.entity.type.StockItem import StockItem


class Item(Agenda, AddParameterTrait):
    _ref_elements = ['typeServiceMOSS', 'centre', 'activity', 'contract']
    _elements = ['text', 'quantity', 'delivered', 'unit', 'coefficient', 'payVAT', 'rateVAT', 'percentVAT',
                 'discountPercentage', 'homeCurrency', 'foreignCurrency', 'note', 'code', 'stockItem', 'centre',
                 'activity', 'contract', 'PDP']

    def __init__(self, data: dict, ico: str):
        # process home currency
        home_currency = data.get('homeCurrency')
        if home_currency:
            data['homeCurrency'] = CurrencyItem(home_currency, ico)

        # process foreign currency
        foreign_currency = data.get('foreignCurrency')
        if foreign_currency:
            data['foreignCurrency'] = CurrencyItem(foreign_currency, ico)

        # process stock item
        stock_item = data.get('stockItem')
        if stock_item:
            data['stockItem'] = StockItem(stock_item, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('orderItem', namespace='ofr')
        self._add_elements(xml, self._elements + ['parameters'], 'ofr')
        return xml



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.type.CurrencyForeign import CurrencyForeign
from pohoda.entity.type.CurrencyHome import CurrencyHome


class Summary(Agenda):
    _elements = ['roundingDocument', 'roundingVAT', 'homeCurrency', 'foreignCurrency']

    def __init__(self, data: dict, ico: str):
        # process home currency
        home_currency = data.get('homeCurrency')
        if home_currency:
            data['homeCurrency'] = CurrencyHome(home_currency, ico)

        # process foreign currency
        foreign_currency = data.get('foreignCurrency')
        if foreign_currency:
            data['foreignCurrency'] = CurrencyForeign(foreign_currency, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('prijemkaSummary', namespace='pri')
        self._add_elements(xml, self._elements, 'pri')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.print_request.PrinterSettings import PrinterSettings
from pohoda.entity.print_request.Record import Record


class PrintRequest(Agenda):

    def __init__(self, data: dict, ico: str):
        # process record
        data['record'] = Record(data['record'], ico, )
        # process printer settings
        data['printerSettings'] = PrinterSettings(data['printerSettings'], ico)
        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('print', namespace='prn')
        xml.set('version', '1.0')
        self._add_elements(xml, ['record', 'printerSettings'], 'prn')
        return xml



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.addressbook.Header import Header
from pohoda.entity.common.AddActionTypeTrait import AddActionTypeTrait
from pohoda.entity.common.AddParameterToHeaderTrait import AddParameterToHeaderTrait


class Addressbook(Agenda, AddActionTypeTrait, AddParameterToHeaderTrait):
    import_root = 'lAdb:addressbook'

    def __init__(self, data: dict, ico: str):
        if data:
            data = {
                'header': Header(data, ico)
            }
        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('addressbook', namespace='adb')
        xml.set('version', '2.0')
        self._add_elements(xml, ['actionType', 'header'], 'adb')
        return xml



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class Pdf(Agenda):
    _elements = ['fileName']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('pdf', namespace='prn')
        self._add_elements(xml, self._elements, 'prn')
        return xml



from typing import List
from importlib import import_module
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.document.Part import Part
from pohoda.entity.common.AddParameterToHeaderTrait import AddParameterToHeaderTrait


class Document(Agenda, AddParameterToHeaderTrait):

    def __init__(self, data: dict, ico: str):
        self._ico = ico

        if data:
            data = {
                'header': self._get_document_part('Header', data, ico)
            }

        self._data = data

        super().__init__(data, ico)

    def _get_document_name(self) -> str:
        raise NotImplementedError

    def _get_document_namespace(self) -> str:
        raise NotImplementedError

    def _get_document_elements(self) -> List[str]:
        return ['header', '{}Detail'.format(self._get_document_name()), 'summary']

    def _get_document_part(self, name: str, data: dict, ico: str) -> Part:
        part_name = 'pohoda.entity.{}.{}'.format(self.__class__.__name__.lower(), name)

        try:
            entity = getattr(import_module(part_name), name)
            if not issubclass(entity, Part):
                raise ValueError('Not allowed entity: {}'.format(name))
            part = entity(data, ico)
            part.set_namespace(self._get_document_namespace())
            part.set_node_prefix(self._get_document_name())
            return part
        except ImportError as e:
            raise ValueError('Entity {} not found'.format(name)) from e

    def add_item(self, data: dict) -> 'Document':
        key = '{}Detail'.format(self._get_document_name())
        if key not in self._data:
            self._data[key] = []

        self._data[key].append(self._get_document_part('Item', data, self._ico))

        return self

    def add_summary(self, data: dict) -> 'Document':
        self._data['summary'].append(self._get_document_part('Summary', data, self._ico))

        return self

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag(self._get_document_name(), namespace=self._get_document_namespace())
        xml.set('version', '2.0')
        self._add_elements(xml, self._get_document_elements(), self._get_document_namespace())
        return xml



from setuptools import setup, find_packages


setup(
    name='pohoda',
    version='1.17.3',
    packages=find_packages(exclude=['tests', 'tests.*']),
    package_data={'pohoda': ['py.typed']},
    install_requires=[
        'lxml',
        'pyhumps'
    ],
    tests_require=[
        'tox'
    ],
    url='https://github.com/Salamek/pohoda',
    license='LGPL-3.0 ',
    author='Adam Schubert',
    author_email='adam.schubert@sg1-game.net',
    description='Python library for generating pohoda XML files',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    test_suite='tests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development',
    ],
    python_requires='>=3.4',
    project_urls={
        'Release notes': 'https://github.com/Salamek/pohoda/releases',
    },
)



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class IndividualPrice(Agenda):
    import_root = 'lst:individualPrice'

    def get_xml(self) -> etree.Element:
        raise NotImplementedError('Individual prices import is currently not supported.')



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.AddParameterTrait import AddParameterTrait
from pohoda.entity.type.Address import Address


class Desc(Agenda, AddParameterTrait):
    _ref_elements = ['number', 'responsiblePerson']

    _elements = ['number', 'datePlanStart', 'datePlanDelivery', 'dateStart', 'dateDelivery', 'dateWarranty', 'text',
                 'partnerIdentity', 'responsiblePerson', 'note']

    def __init__(self, data: dict, ico: str):
        # process partner identity
        partner_identity = data.get('partnerIdentity')
        if partner_identity:
            data['partnerIdentity'] = Address(partner_identity, ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('contractDesc', namespace='con')
        self._add_elements(xml, self._elements + ['parameters'], 'con')
        return xml



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class RecyclingContrib(Agenda):
    _elements = ['recyclingContribType', 'coefficientOfRecyclingContrib']
    _ref_elements = ['recyclingContribType']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('recyclingContrib', namespace='stk')
        self._add_elements(xml, self._elements, 'stk')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda


class Category(Agenda):
    import_root = 'ctg:category'

    _elements = ['name', 'description', 'sequence', 'displayed', 'picture', 'note']

    def add_subcategory(self, category: 'Category') -> None:
        """
        Add subcategory.
        :param category:
        :return:
        """

        if not self._data.get('subCategories'):
            self._data['subCategories'] = []

        self._data['subCategories'].append(category)

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('categoryDetail', namespace='ctg')
        xml.set('version', '2.0')
        self.category_xml(xml)
        return xml

    def category_xml(self, xml: etree.Element) -> None:
        """
        Attach category to XML element.
        :param xml:
        :return:
        """

        category = self._create_xml_tag('category', namespace='ctg')
        xml.append(category)
        self._add_elements(category, self._elements, 'ctg')

        sub_categories = self._data.get('subCategories')
        if sub_categories:
            sub_categories_element = self._create_xml_tag('subCategories', namespace='ctg')
            category.append(sub_categories_element)
            for sub_category in sub_categories:
                sub_category.category_xml(sub_categories_element)



# coding: utf-8

from lxml import etree
from pohoda.entity.Agenda import Agenda


class Settings(Agenda):
    _ref_elements = ['currency']
    _elements = ['unit', 'length', 'currency', 'parameterList']

    def get_xml(self) -> etree.Element:
        xml = self._create_xml_tag('parameterSettings', namespace='ipm')
        self._add_elements(xml, self._elements, 'ipm')
        return xml



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda


class ActionPrice(Agenda):
    import_root = 'lst:actionPrice'

    def get_xml(self) -> etree.Element:
        raise NotImplementedError('Action prices import is currently not supported.')



# coding: utf-8
from lxml import etree
from pohoda.entity.Agenda import Agenda
from pohoda.entity.common.SetNamespaceTrait import SetNamespaceTrait
from pohoda.entity.common.SetNodeNameTrait import SetNodeNameTrait
from pohoda.entity.type.AddressInternetType import AddressInternetType
from pohoda.entity.type.EstablishmentType import EstablishmentType


class MyAddress(Agenda, SetNamespaceTrait, SetNodeNameTrait):
    _elements = ['address', 'establishment']

    def __init__(self, data: dict, ico: str):
        address = data.get('address')
        if address:
            data['address'] = AddressInternetType(address, ico)

        establishment = data.get('establishment')
        if establishment:
            data['establishment'] = EstablishmentType(data['establishment'], ico)

        super().__init__(data, ico)

    def get_xml(self) -> etree.Element:
        if not self._namespace:
            raise ValueError('Namespace not set.')

        if not self._node_name:
            raise ValueError('Node name not set.')

        xml = self._create_xml_tag(self._node_name, namespace=self._namespace)
        self._add_elements(xml, self._elements, 'typ')
        return xml
